@@ -18,6 +18,7 @@
 #include <locale.h>
 #include <libebook/libebook.h>
 
+#include "e-test-server-utils.h"
 #include "data-test-utils.h"
 
 static EbSqlClosure closure = { FALSE, NULL };
@@ -77,8 +78,8 @@ test_create_cursor_invalid_sort (EbSqlFixture *fixture,
 		sort_fields, sort_types, 1, &error);
 
 	g_assert (cursor == NULL);
-	g_assert (error);
-	g_assert (g_error_matches (error, E_BOOK_SQLITE_ERROR, E_BOOK_SQLITE_ERROR_INVALID_QUERY));
+	g_assert_error (error, E_BOOK_SQLITE_ERROR, E_BOOK_SQLITE_ERROR_INVALID_QUERY);
+	g_clear_error (&error);
 }
 
 static void
@@ -91,8 +92,8 @@ test_create_cursor_missing_sort (EbSqlFixture *fixture,
 	cursor = e_book_sqlite_cursor_new (fixture->ebsql, NULL, NULL, NULL, 0, &error);
 
 	g_assert (cursor == NULL);
-	g_assert (error);
-	g_assert (g_error_matches (error, E_BOOK_SQLITE_ERROR, E_BOOK_SQLITE_ERROR_INVALID_QUERY));
+	g_assert_error (error, E_BOOK_SQLITE_ERROR, E_BOOK_SQLITE_ERROR_INVALID_QUERY);
+	g_clear_error (&error);
 }
 
 gint
@@ -121,5 +122,5 @@ main (gint argc,
 		"/EbSqlCursor/Create/MissingSort", EbSqlFixture, &closure,
 		e_sqlite_fixture_setup, test_create_cursor_missing_sort, e_sqlite_fixture_teardown);
 
-	return g_test_run ();
+	return e_test_server_utils_run_full (0);
 }
